﻿/*
 * LCD_keyb_ROT.h
 *
 * Created: 20/09/2016 1:35:06 am
 * Author:	Dimitrios Porlidas
 *
 * A complete program that recognizes input from a 4x4 keyboard and from a ROT and displays keyboard' s characters 
 * and ROT' s counting in a LCD 2x16 or 4x20. LCD can be connected to any port (PORTx1:7), while PORTx1 is the R/W, 
 * PORTx2 is the RS, PORTx3 is the En signal and PORTx4:7 are D4:7 of LCD. Keyboard 4x4 can be connected to any 
 * port (PORTx0:7), while PORTx0:3 are outputs to rows and PORTx4:7 are inputs from columns. By default LCD's port 
 * is PORTA and keyboard's port is PORTB, but both ports can be changed by changing the definitions a few lines below. 
 * ROT' s implementation is with interrupts and has to be connected to INT0 (PORTD2), INT1 (PORTD3). 
 * The program tested with ATmega16A at 8000000MHz with external crystal oscillator and AtmelStudio 7.0.
 *
 */ 

#ifndef F_CPU
#define F_CPU 8000000UL
#endif

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>									// For "sei()", "cli()" functions
#include <stdlib.h>											// For "itoa" function
#include <string.h>											// For "strlen" function

#ifndef LCD_KEYB_ROT_H_
#define LCD_KEYB_ROT_H_

uint8_t entry;												// bit0 keyb.c, bit1 USART.c
uint8_t data;												// keyb.c
uint8_t ln1;												// keyb.c
uint8_t line;												// keyb.c
uint8_t out;												// LCD.c
uint8_t lg1;												// LCD.c
volatile int reg1bin;										// ROT.c Has to be "volatile" if other routine than ISR(INTx_vect) affects "reg1bin"
char reg1asc[6];											// ROT.c
uint8_t rsdata;												// USART.c

#define lcdport PORTA										// Define LCD PORT
#define lcdddr DDRA											// Define LCD DDR
#define lcdreg PINA											// Define LCD PIN 
#define keyport PORTB										// Define keyboard PORT
#define keyddr DDRB											// Define keyboard DDR
#define keyreg PINB											// Define keyboard PIN

#define CurL 0b00010000 									// Cursor Left (instruction)
#define CurR 0b00010100 									// Cursor Right (instruction)
#define DisL 0b00011000 									// Display Left (instruction)
#define DisR 0b00011100 									// Display Right (instruction)
#define CDRC 0b00000001 									// Clear Display and Reset Cursor (instruction, use 2ms delay after)
#define RDRC 0b00000010										// Reset Display and Reset Cursor (instruction, use 2ms delay after)

#define Da 0b00000100										// RS Register Select, data signal (0 for instruction)
#define Rd 0b00000010										// R/W Read/Write from/to LCD signal (0 for write)
#define En 0b00001000										// Enable signal

void WrIn (uint8_t tmp);       								// Write Instruction routine
void WrDa (uint8_t tmp);									// Write Data routine
void SeCu (uint8_t addr);									// Send cursor to a specific address (0-39, 64-103) or a specific assignment (L1R1-L4R20)
void SeCG (uint8_t addr);									// Set CGRAM address
void TypeLogo (uint8_t logo2[]);		 					// Logo routine
void InitLCD (void);										// Start up LCD, initialization routine
void TypeMainLogo (void);									// Main Logo routine
void Initkeyb (void);										// Set up 4x4 keyboard
void Keyb (void);											// Keyboard routine
void InitROT (void);										// ROT routine
void CountROT (void);										// ROT routine
void USART_Init(unsigned int baud);							// USART.c
void USART_Transmit(unsigned char trnsm);					// USART.c
unsigned char USART_Receive(void);							// USART.c
//ISR(INT0_vect);
//ISR(INT1_vect);

#define pm 0x10			// plus minus
#define ae 0x1A			// almost equal
#define ti 0x1D			// middle ~
#define t2 0x1E			// ^2
#define t3 0x1F			// ^3
#define po 0xA5			// £
#define ye 0xA6			// yen
#define pt 0xA7			// Pt
#define fm 0xA8			// f(math)
#define zm 0xAD			// 0(math)
#define it 0xB5			// i2
#define dv 0xB8			// divide
#define se 0xB9			// less or equal
#define be 0xBA			// great or equal
#define dl 0xBB			// <<
#define dg 0xBC			// >>
#define ne 0xBD			// not equal
#define ro 0xBE			// root
#define in 0xC2			// infinity
#define re 0xC4			// return
#define au 0xC5			// up
#define ad 0xC6			// down
#define ar 0xC7			// right
#define al 0xC8			// left
#define tm 0xD0			// trade mark
#define pg 0xD2			// paragraph
#define gG 0xD4			// gama cap
#define gD 0xD5			// delta cap
#define gU 0xD6			// theta cap
#define gL 0xD7			// lamda cap
#define gJ 0xD8			// ksi cap
#define gP 0xD9			// pi cap
#define gS 0xDA			// sigma cap
#define gT 0xDB			// taf cap
#define gF 0xDC			// fi cap
#define gC 0xDD			// psi cap
#define gV 0xDE			// omega cap
#define ga 0xDF			// alpa
#define gb 0xE0			// beta
#define gg 0xE1			// gama
#define gd 0xE2			// delta
#define ge 0xE3			// epsilon
#define gz 0xE4			// zita
#define gh 0xE5			// ita
#define gu 0xE6			// theta
#define gi 0xE7			// iota
#define gk 0xE8			// kapa
#define gl 0xE9			// lamda
#define gm 0xEA			// mi
#define gn 0xEB			// ni
#define gj 0xEC			// ksi
#define gp 0xED			// pi
#define gr 0xEE			// ro
#define gs 0xEF			// sigma
#define gt 0xF0			// taf
#define gy 0xF1			// ypsilon
#define gf 0xF2			// fi
#define gv 0xF3			// psi
#define gw 0xF4			// omega
#define dn 0xF5			// down
#define ri 0xF6			// right
#define le 0xF7			// left

#define L1R1	0x00	// LCD 2x16 (1602) & 4x20 (2004) 1st line 1st rank
#define L1R2	0x01
#define L1R3	0x02
#define L1R4	0x03
#define L1R5	0x04
#define L1R6	0x05
#define L1R7	0x06
#define L1R8	0x07
#define L1R9	0x08
#define L1R10	0x09
#define L1R11	0x0A
#define L1R12	0x0B
#define L1R13	0x0C
#define L1R14	0x0D
#define L1R15	0x0E
#define L1R16	0x0F
#define L1R17	0x10
#define L1R18	0x11
#define L1R19	0x12
#define L1R20	0x13

#define L2R1	0x40	// LCD 2x16 (1602) & 4x20 (2004) 2nd line 1st rank
#define L2R2	0x41
#define L2R3	0x42
#define L2R4	0x43
#define L2R5	0x44
#define L2R6	0x45
#define L2R7	0x46
#define L2R8	0x47
#define L2R9	0x48
#define L2R10	0x49
#define L2R11	0x4A
#define L2R12	0x4B
#define L2R13	0x4C
#define L2R14	0x4D
#define L2R15	0x4E
#define L2R16	0x4F
#define L2R17	0x50
#define L2R18	0x51
#define L2R19	0x52
#define L2R20	0x53

#define L3R1	0x14	// LCD 2x16 (1602) 1st line 21st rank & 4x20 (2004) 3rd line 1st rank
#define L3R2	0x15
#define L3R3	0x16
#define L3R4	0x17
#define L3R5	0x18
#define L3R6	0x19
#define L3R7	0x1A
#define L3R8	0x1B
#define L3R9	0x1C
#define L3R10	0x1D
#define L3R11	0x1E
#define L3R12	0x1F
#define L3R13	0x20
#define L3R14	0x21
#define L3R15	0x22
#define L3R16	0x23
#define L3R17	0x24
#define L3R18	0x25
#define L3R19	0x26
#define L3R20	0x27

#define L4R1	0x54	// LCD 2x16 (1602) 2nd line 21st rank & 4x20 (2004) 4th line 1st rank
#define L4R2	0x55
#define L4R3	0x56
#define L4R4	0x57
#define L4R5	0x58
#define L4R6	0x59
#define L4R7	0x5A
#define L4R8	0x5B
#define L4R9	0x5C
#define L4R10	0x5D
#define L4R11	0x5E
#define L4R12	0x5F
#define L4R13	0x60
#define L4R14	0x61
#define L4R15	0x62
#define L4R16	0x63
#define L4R17	0x64
#define L4R18	0x65
#define L4R19	0x66
#define L4R20	0x67

#endif /* LCD_KEYB_ROT_H_ */
